#ifndef BITplayerH
#define BITplayerH

#include <string>
#include <istream>

#include "Pod.h"

namespace JTAG
{

#if defined(__WIN32__) || defined(_WIN32)
  #if defined(__DLL__)
	#define LINKAGE __declspec(dllexport)
  #else
	#define LINKAGE __declspec(dllimport)
	#pragma link "JTAGLib.lib"
  #endif
#else
  #define LINKAGE
#endif

struct ChainInfo;

class LINKAGE BITPlayer
	{
public:

	static BITPlayer* new_BITPlayer
		(Pod&	pod,
		 Pod::TAPPort	port);

	virtual ~BITPlayer
		();

	typedef bool (*StatusUpdate)(unsigned at, unsigned of);

	/// Play: play an BIT file to the pod on the selected port
	//	 Returns:
	//		nothing, but will have called the error function if
	//		there was an error
	//	 Parameters:
	//		script			the BIT binary
	//		chain_layout	optional chain structure record (see below)
	//      status_func		callback made every statement or ~4K bits scanned
	//		device			Type of device - don't use Auto in chain

	//!!! N.b.: it's not clear to me yet whether all Xilinx FPGAs have the
	//!!! same shutdown, configuration, startup sequence

	void Play
		(std::istream&		script,
		 ChainInfo*			chain_layout = 0,
		 StatusUpdate		status_func = 0);

	// The chain structure record if present has the following effects:
	//	1.	The IR bits before and after the device are set to BYPASS
	//	2.	The additional DR Bypass register bits are taken into account
	//	3.	Appropriate adjustments to tbe bitstream as described in
	//		XAPP188 are made.

protected:
	BITPlayer
		(Pod&, Pod::TAPPort);

private:
	class Implementation;
	Implementation*	impl;
	};

#undef LINKAGE

}

#endif
