//---------------------------------------------------------------------------

#include <memory>
#pragma hdrstop

#include "BitString.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

using namespace std;

namespace JTAG
{

BitComposer::BitComposer(unsigned buffer_bytes)
	:	current(0),
		out_shift(0),
		hold(0)
{
	buffer.resize(buffer_bytes);
	// limit normal output to buffer size - 1 so that the overflow (hold)
	// bits may always be entered into the output stream
	limit = buffer.end() - 1;
	outp = buffer.begin();
}

inline void BitComposer::next_word()
{
	if (words)
		{ --words; word = *--pos; }
	else
		{ word = empty; }
}

bool BitComposer::Compose
	(unsigned a_bits, bool care, BitString* bs)	// true when input exhausted
{
	if (bs != current)
		{
		current = bs;
		in_shift = bs->shift;
		pos = bs->data.end();
		bits = a_bits;
		words = bs->data.size();
		empty = BitString::Base_t(care ? -1 : 0);
		next_word();
		}

	while ((bits != 0) && (outp != limit))
		{
		// extract next byte
		Byte x;
		switch (in_shift)
			{
		case BitString::MAX_SHIFT:
			// new word
			next_word();
			x = Byte(word);
			in_shift = 8;
			break;

		case BitString::MAX_SHIFT - 4:
			// split byte
			x = word >> in_shift;
			next_word();
			x |= Byte(word & 0xF);
			in_shift = 4;
			break;

		default:
			x = Byte(word >> in_shift);
			in_shift += 8;
			}

		// 3 cases handled here:
		//	(a) held bits + new bits are less than a byte's worth
		//	(b) held bits + new bits are more than a byte but less than two bytes
		//	(c) held bits + new bits are 2 bytes or more
		if (bits + out_shift < 8)
			{
			// (a) less than a byte buffered, hold it
			hold |= (x << out_shift);
			out_shift += bits;
			return true;
			}
		else
			{
			// equal or more than a byte - deposit the low byte into
			// the output buffer and hold the upper
			unsigned temp = (x << out_shift) | hold;;
			*outp++ = Byte(temp);
			hold = Byte(temp >> 8);
			if ((bits -= 8) <= 0)	// (b)
				{
				out_shift += bits;
				return true;
				}
			}
		}

	return (bits <= 0);
}

void BitComposer::Flush()
{
	if (out_shift > 0)
		{
		*outp++ = hold;
		out_shift = 0;
		}
}

}

