//---------------------------------------------------------------------------

#ifndef BitStringH
#define BitStringH

#include <vector>

namespace JTAG
{

#if defined(__WIN32__) || defined(_WIN32)
  #if defined(__DLL__)
	#define LINKAGE __declspec(dllexport)
  #else
	#define LINKAGE __declspec(dllimport)
	#pragma link "JTAGLib.lib"
  #endif
#else
  #define LINKAGE
#endif

class LINKAGE BitString
	{
public:
	typedef unsigned Base_t;
	enum { MAX_SHIFT = 8*sizeof(Base_t) };

	bool 		present;
	unsigned	shift;
	std::vector<Base_t> data;

	BitString() : present(false) {}
	void reset();
	};

class LINKAGE BitComposer
	{

public:

	typedef unsigned char Byte;
	typedef std::vector<Byte> Buffer_t;

	Buffer_t			buffer;
	Buffer_t::iterator	limit;
	Byte*				outp;

	BitComposer(unsigned buffer_bytes);
	virtual ~BitComposer() {}

	bool Compose(unsigned bits, bool care, BitString*);	// true when input exhausted
	void Clear_Buffer();
	void Flush();

private:
	BitString*			current;
	BitString::Base_t	*pos, word, empty;
	int					bits, words, in_shift, out_shift;
	Byte				hold;

	void next_word();
	};

//-- implementation --

inline void BitString::reset()
{
	data.clear();
	present = false;
}

inline void BitComposer::Clear_Buffer()
{
	outp = buffer.begin();
}

#undef LINKAGE

}

#endif
