#ifndef ChainH
#define ChainH

#include <string>
#include <vector>
#include <iostream>

namespace JTAG
{

#if defined(__WIN32__) || defined(_WIN32)
  #if defined(__DLL__)
	#define LINKAGE __declspec(dllexport)
  #else
	#define LINKAGE __declspec(dllimport)
	#pragma link "JTAGLib.lib"
  #endif
#else
  #define LINKAGE
#endif

class Pod;

typedef unsigned IDCODE_t;

class ChainInfo
	{
public:

	unsigned	devices_before;
	unsigned	IR_bits_before;
	IDCODE_t	IDCODE;
	unsigned	IR_bits;
	unsigned	devices_after;
	unsigned	IR_bits_after;

	ChainInfo(unsigned IDCODE = 0);
	};

class LINKAGE Device
	{
public:
	IDCODE_t		IDCODE;
	IDCODE_t		IDCODE_mask;
	unsigned		IR_bits;
	unsigned		IDCODE_instruction;
	std::string		dev_name;
	std::string		dev_description;
	std::string		BSDL_file;
	float			max_freq;

	Device(IDCODE_t IDCODE = 0);

	void Store(std::ostream&);
	void Load(std::istream&);
	};

class LINKAGE Device_Database
	{
public:
	std::vector<Device>		devices;

	Device* Find(IDCODE_t IDCODE);
	Device* Create(IDCODE_t IDCODE, unsigned IR_bits);

	void Store(std::ostream&);
	void Load(std::istream&);
	};

class LINKAGE Chain
	{
public:

	std::vector<Device*>		devices;
	std::vector<std::string>	data_files;
	std::vector<IDCODE_t>		IDCODEs;

	typedef Device* (*ResolveCB)(Chain&, unsigned pos);

	bool ValidateChain(Pod&, unsigned port);
	bool ReadChain(Pod&, unsigned port, Device_Database* = 0);
	bool ResolveDevices(Device_Database* = 0, ResolveCB = 0);
	void GetChainInfo(unsigned position, ChainInfo&);

	void Store(std::ostream&);
	void Load(std::istream&);
	};

#undef LINKAGE

}

#endif

