#ifndef SVFplayerH
#define SVFplayerH

#include <string>
#include <istream>

#include "Pod.h"

namespace JTAG
{

#if defined(__WIN32__) || defined(_WIN32)
  #if defined(__DLL__)
	#define LINKAGE __declspec(dllexport)
  #else
	#define LINKAGE __declspec(dllimport)
	#pragma link "JTAGLib.lib"
  #endif
#else
  #define LINKAGE
#endif

struct ChainInfo;

class LINKAGE SVFPlayer
	{
public:

	static SVFPlayer* new_SVFPlayer
		(Pod&	pod,
		 Pod::TAPPort	port);

	virtual ~SVFPlayer
		();

	typedef bool (*StatusUpdate)(unsigned at, unsigned of);

	/// Play: play an SVF file to the pod on the selected port
	//	 Returns:
	//		nothing, but will have called the error function if
	//		there was an error
	//	 Parameters:
	//		script			the SVF text
	//		chain_layout	optional chain structure record (see below)
	//      status_func		callback made every statement or ~4K bits scanned
	//		Xilinx_timing	TCKs in RUNTEST really means 500KHz cycles
	//						(why didn't they use [n SEC]?)
	//		check_TDO		if false, skip TDO/TDOMask checking

	void Play
		(std::istream&		script,
		 ChainInfo*			chain_layout = 0,
		 StatusUpdate		status_func = 0,
		 bool				Xilinx_timing = true,
		 bool				check_TDO = true);

	// The chain structure record if present has the following effects:
	//	1.	The HIR and TIR commands are fixed at BYPASS for the sizes indicated
	//	2.	The HDR and TDR commands are fixed at 0's for the sizes indicated
	//	3.	SIR commands of different sizes than the selected device are not
	//		executed.
	//	4.	TDR commands are only executed when they follow an SIR that was
	//		executed.
	//	These rules are intended to execute only the portions of an SVF file
	//	that apply to the targeted device, though not all SVF's will be
	//	formatted in a way that makes this possible.

protected:
	SVFPlayer
		(Pod&, Pod::TAPPort);

private:
	class Implementation;
	Implementation*	impl;
	};

#undef LINKAGE

}

#endif

