#if !defined(SIMPLELEXER_H)
#define SIMPLELEXER_H

#include <istream>
#include <string>
#include <vector>
#include <set>

namespace JTAG
{

#if defined(__WIN32__) || defined(_WIN32)
  #if defined(__DLL__)
	#define LINKAGE __declspec(dllexport)
  #else
	#define LINKAGE __declspec(dllimport)
	#pragma link "JTAGLib.lib"
  #endif
#else
  #define LINKAGE
#endif

class LINKAGE SimpleLexer
	{
public:

	typedef char Char_t;
	typedef std::basic_string<Char_t> Keyword;
	typedef std::vector<Keyword*> KeywordList;
	typedef std::set<Char_t> SetOfChar;

	static SetOfChar UCALPHA;
	static SetOfChar NUMBERS;
	static SetOfChar UCALPHANUM;

	SimpleLexer
		(const Keyword*		keywords_begin,
		 const Keyword*		keywords_end,
		 bool				case_sensitive = false,
		 const SetOfChar&	ident_start = UCALPHA,
		 const SetOfChar&	ident_rest = UCALPHANUM);

	enum LexerResult
		{
		rNone,
		rUnknown,
		rEOL,
		rEOF,
		rWhite,
		rKeyword,
		rIdent,
		rNumber
		};

	void Open(std::istream&	input);
	unsigned tellg();

	LexerResult Peek();
	LexerResult Next();

	int PeekChar();
	void EatPeekChar();
	int NextChar();
	void SkipWS();

	int				line, column;

	int				index;
	std::string		stringVal;
	int				intVal;

private:
	LexerResult Lex1();
	void check_buffer();
	int peek2();

	Char_t			*cur, *end;
	bool			last_buffer;
	unsigned		already_read;

	LexerResult		last;
	std::istream*	input;
	const bool		case_sensitive;
	const SetOfChar& ident_start;
	const SetOfChar& ident_rest;

	class Data;
	Data*			data;
	};

#undef LINKAGE

}

#endif


