//---------------------------------------------------------------------------

#include <vcl.h>
#include <vector>
#include <strstream>
#include <fstream>
#pragma hdrstop

#include "APITest.h"
#include "ChainForm.h"
#include "Program.h"
#include "SVFplayer.h"
#include "BITplayer.h"
#include "Progress.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
	Application->OnException = Report_Exception;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
	// rescan USB devices
	ButtonRefreshClick(Sender);

	ComboBoxPulseTo->ItemIndex = 0;
}
//---------------------------------------------------------------------------

static void EnableControls
	(TWinControl* p, bool enable)
{
	if (p)
		{
		for (int n = 0; n < p->ControlCount; ++n)
			{
			// only those with Tag != 0 are enabled or disabled
			if (p->Controls[n]->Tag)
				{
				EnableControls(dynamic_cast<TWinControl*>(p->Controls[n]), enable);
				p->Controls[n]->Enabled = enable;
				}
			}
		}
}

void __fastcall TForm1::ButtonOpenCloseClick(TObject *Sender)
{
	Screen->Cursor = crHourGlass;
	bool enable = false;
	if (tm.get())
		{
		try
			{
			tm.reset(0);
			}
		catch (const JTAG::Error_Exception& err)
			{
			MessageDlg(err.what(), mtError, TMsgDlgButtons() << mbOK, 0);
			}
		}
	else
		{
		try
			{
			tm.reset(JTAG::new_Pod(ComboBoxPod->ItemIndex));
			tm->Set_LED(0, ButtonLED1->Down);
			tm->Set_LED(1, ButtonLED2->Down);
			tm->Set_LED(2, ButtonLED3->Down);
			tm->Set_LED(3, ButtonLED4->Down);
			enable = true;
			}

		catch (const JTAG::Error_Exception& msg)
			{
			Application->MessageBox(msg.what(), "Open Error!", MB_OK);
			}
		}

	// enable/disable all child controls except the few we need to stay enabled
	EnableControls(this, enable);

	if (enable)
		{
		ButtonOpenClose->Caption = "Close";
		ButtonRefresh->Enabled = false;
		ButtonProgram->Enabled = false;
		ComboBoxPod->Enabled = false;

		// Stock TCK speed combo
		tm->Get_General_Info(general);
		ComboBoxClock->Clear();
		for (unsigned n = general.num_TCK_rates; n--;)
			{
			char buf[20];
			std::ostrstream fmt(buf, sizeof(buf));

			if (general.TCK_rates[n] < 1e6)
				fmt << general.TCK_rates[n] / 1e3 << " KHz";
			else
				fmt << general.TCK_rates[n] / 1e6 << " MHz";
			fmt << '\0';

			ComboBoxClock->Items->Insert(0, fmt.str());
			}

		ButtonTAP1->Down = true;
		ButtonTAP2->Down = false;
		CheckBoxTDOClick(Sender);
		updateTAPInfo();
		Timer1->Enabled = true;
		}
	else
		{
		ButtonOpenClose->Caption = "Open";
		ButtonRefresh->Enabled = true;
		ButtonProgram->Enabled = true;
		ComboBoxPod->Enabled = true;
		ButtonRefreshClick(Sender);
		}
	Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonRefreshClick(TObject *Sender)
{
	unsigned count = 0;
	JTAG::CountDevices(count);

	ButtonOpenClose->Enabled = count > 0;
	ComboBoxPod->Clear();
	while (count > 0)
		ComboBoxPod->Items->Insert(0, IntToStr(--count));
	ComboBoxPod->ItemIndex = 0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonLEDClick(TObject *Sender)
{
	TSpeedButton* button = dynamic_cast<TSpeedButton*>(Sender);
	tm->Set_LED(button->Tag - 1, button->Down);
	tm->Flush_Commands();
}
//---------------------------------------------------------------------------

static JTAG::Pod::GPIO lu[] =
	{ JTAG::Pod::is0, JTAG::Pod::is1, JTAG::Pod::isZ };

static int xlu(JTAG::Pod::GPIO g)
{
	switch (g)
		{
	case JTAG::Pod::is0: return 0;
	case JTAG::Pod::is1: return 1;
	default : return 2;
		}
}

bool __fastcall TForm1::updateGPIOs()
{
	JTAG::Pod::GPIO value[4];
	for (int n = 4; n--;)
		{
		try
			{
			tm->Get_TAP_GPIO_Input(port, n, value[n]);
			}

		catch (JTAG::Error_Exception&)
			{
			Timer1->Enabled = false;
			Application->MessageBox("Lost contact with the pod!", "Problem!", MB_OK);
			ButtonOpenCloseClick(this);
			return false;
			}
		}
	static char text[] = "Z0Z1";

	Display1->Text = text[value[0]];
	Display2->Text = text[value[1]];
	Display3->Text = text[value[2]];
	Display4->Text = text[value[3]];
	return true;
}

void __fastcall TForm1::updateTAPInfo()
{
	port = int(ButtonTAP2->Down);
	tm->Get_TAP_Settings(port, info);

	JTAG::Pod::TAP_Capabilities info2;
	tm->Get_TAP_Info(port, info2);

	CheckBoxTristated->Checked = info.tristated && info2.canTristate;
	CheckBoxTristated->Enabled = info2.canTristate;

	//current_TDO_buffer;
	ComboBoxClock->ItemIndex = info.TCK_rate_index;

	ComboBoxTCKTerm->ItemIndex = info.TCK_term;
	ComboBoxTCKTerm->Enabled = info2.hasTCKTerm;

	ComboBoxTDOTerm->ItemIndex = info.TDO_term;
	ComboBoxTDOTerm->Enabled = info2.hasTDOTerm;

	ComboBoxGPIO1->ItemIndex = xlu(info.Current_GPIO_Setting[0]);
	ComboBoxGPIO2->ItemIndex = xlu(info.Current_GPIO_Setting[1]);
	ComboBoxGPIO3->ItemIndex = xlu(info.Current_GPIO_Setting[2]);
	ComboBoxGPIO4->ItemIndex = xlu(info.Current_GPIO_Setting[3]);

	updateGPIOs();
}

void __fastcall TForm1::ButtonTAP1Click(TObject *Sender)
{
	updateTAPInfo();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ComboBoxClockChange(TObject *Sender)
{
	info.TCK_rate_index = ComboBoxClock->ItemIndex;
	tm->Set_TAP_Settings(port, info);
	tm->Flush_Commands();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ComboBoxTCKTermChange(TObject *Sender)
{
	info.TCK_term = JTAG::Pod::TAP_Settings::TCK_Terminations(ComboBoxTCKTerm->ItemIndex);
	tm->Set_TAP_Settings(port, info);
	tm->Flush_Commands();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ComboBoxTDOTermChange(TObject *Sender)
{
	info.TDO_term = JTAG::Pod::TAP_Settings::TDO_Terminations(ComboBoxTDOTerm->ItemIndex);
	tm->Set_TAP_Settings(port, info);
	tm->Flush_Commands();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CheckBoxTristatedClick(TObject *Sender)
{
	info.tristated = CheckBoxTristated->Checked;
	tm->Set_TAP_Settings(port, info);
	tm->Flush_Commands();
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
	if (tm.get())
		updateGPIOs();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ComboBoxGPIOChange(TObject *Sender)
{
	TComboBox* combo = dynamic_cast<TComboBox*>(Sender);
	tm->Set_TAP_GPIO(port, combo->Tag - 1, lu[combo->ItemIndex]);
	tm->Flush_Commands();
	updateGPIOs();
}
//---------------------------------------------------------------------------



void __fastcall TForm1::ButtonPulseGPIOClick(TObject *Sender)
{
	TSpeedButton* button = dynamic_cast<TSpeedButton*>(Sender);
	int ns = StrToInt(EditPulseNS->Text);
	JTAG::Pod::GPIO after;
	switch (button->Tag)
		{
	case 1: after = lu[ComboBoxGPIO1->ItemIndex]; break;
	case 2: after = lu[ComboBoxGPIO2->ItemIndex]; break;
	case 3: after = lu[ComboBoxGPIO3->ItemIndex]; break;
	case 4: after = lu[ComboBoxGPIO4->ItemIndex]; break;
		}
	tm->Pulse_TAP_GPIO(port, button->Tag - 1, ns, lu[ComboBoxPulseTo->ItemIndex], after);

}
//---------------------------------------------------------------------------

void __fastcall TForm1::CheckBoxTDOClick(TObject *Sender)
{
	EditTDO->Enabled = CheckBoxTDO->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::EditBinaryFilter(TObject *Sender, char &Key)
{
	if ((Key != '0') && (Key != '1')  && (Key != '\b'))
		{
		Beep();
		Key = '\0';
		}
}
//---------------------------------------------------------------------------

void __fastcall TForm1::EditNumberFilter(TObject *Sender, char &Key)
{
	if (((Key < '0') || (Key > '9')) && (Key != '\b'))
		{
		Beep();
		Key = '\0';
		}
}
//---------------------------------------------------------------------------


void TForm1::ParseBinary(const AnsiString& s, AnsiString& result)
{
	AnsiString res;
	res = AnsiString::StringOfChar('\0', (s.Length() + 7) / 8);
	int idx = 1, bit = 1, last;

	for (int pos = 1; pos <= s.Length(); ++pos)
		{
		last = int(s[pos] == '1');
		if (last)
			res[idx] = char(res[idx] | bit);
		if ((bit <<= 1) > 128)
			bit = 1, ++idx;
		}

	if (last && (bit != 1))
		while (bit <= 128)
			{
			res[idx] = char(res[idx] | bit);
			bit <<= 1;
			}

	result = res;
}

void TForm1::ParseBinary(const AnsiString& s, unsigned& result)
{
	unsigned res = 0;

	if (!s.IsEmpty())
		{
		int pos = 1;
		for (unsigned bit = 1; bit; bit <<= 1)
			{
			if (s[pos] == '1') res |= bit;
			if (pos < s.Length()) ++pos;
			}
		}

	result = res;
}

void __fastcall TForm1::SpeedButtonRunClick(TObject *Sender)
{
	AnsiString TDO, TDI;
	unsigned TMS;
	int bits;

	ParseBinary(EditTMS->Text, TMS);
	ParseBinary(EditTDO->Text, TDO);
	bits = StrToInt(EditLength->Text);

	int bytes = (bits + 7) / 8;

	if (TDO.Length() < bytes)
		TDO += AnsiString::StringOfChar('\0', bytes - TDO.Length());

	if (CheckBoxTDO->Checked)
		tm->Write_TDO_Buffer(port, 0, bytes, TDO.c_str());

	unsigned flags = (TDI.IsEmpty() ? 0 : 1) | (CheckBoxTDO->Checked ? 2 : 0);

	tm->Run_TAP_X(port, bits, TMS, flags);

	if (CheckBoxTDI->Checked)
		{
		TDI = AnsiString::StringOfChar('\0', bytes);
		tm->Read_TDI_Buffer(port, 0, bytes, TDI.c_str());
		}

	AnsiString TDI_res;
	if (TDI.Length())
		{
		int idx = 1, bit = 1;
		for (int b = 0; b < bits; ++b)
			{
			TDI_res += (TDI[idx] & bit) ? '1' : '0';
			if ((bit <<= 1) > 128)
				bit = 1, ++idx;
			}
		}
	EditTDI->Text = TDI_res;

}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonCopyClick(TObject *Sender)
{
	int bytes = (StrToInt(EditLength->Text) + 7) / 8;
	tm->Copy_TDI_Buffer_To_TDO(port, 0, bytes);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonTRSTClick(TObject *Sender)
{
	Timer1->Enabled = false;
	tm->Pulse_TRST(port, StrToInt(EditMS->Text));
	Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonRSTClick(TObject *Sender)
{
	Timer1->Enabled = false;
	tm->Pulse_RST(port, StrToInt(EditMS->Text));
	Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonDINTClick(TObject *Sender)
{
	Timer1->Enabled = false;
	tm->Pulse_DINT(port, StrToInt(EditMS->Text));
	Timer1->Enabled = true;
}
//---------------------------------------------------------------------------



void __fastcall TForm1::SpeedButtonChainClick(TObject *Sender)
{
	// scan identify
	static TStringList* header = NULL;

	if (!header)
		{
		header = new TStringList;
		header->Add("Manufacturer");
		header->Add("Device");
		header->Add("Version");
		Form2->StringGrid1->Rows[0] = header;
		}

	Form2->StringGrid1->RowCount = 1;

	tm->Run_TAP_X(port, 9, 0x05F);

		{
		for (int maxc = 99; maxc--;)
			{
			JTAG::Byte ID[4];

			// N.B. low-byte-first architectures only!
			tm->Do_Shift(port, 32, false, NULL, ID);

			unsigned IDCODE = 0;
			for (int n = 4; n--;) IDCODE = (IDCODE << 8) | ID[n];

			if (!(IDCODE & 1) || (IDCODE == unsigned(-1))) break;

			AnsiString s = IntToHex(int(IDCODE & 0xFFE), 3);
			if ((IDCODE & 0xFFE) == 0x012)
				s += " (Intel)";
			else if ((IDCODE & 0xFFE) == 0x092)
				s += " (Xilinx)";

			TStringList* sl = new TStringList;
			sl->Add(s);

			s = IntToHex(int((IDCODE >> 12) & 0xFFFF), 4);
			switch (IDCODE & 0xFFFFFFF)
				{
			case 0x0610093:	s += " Spartan II 50"; break;
			case 0x0270013: s += " 386ex"; break;
				}
			sl->Add(s);

			s = IntToHex(int((IDCODE >> 28) & 0xF), 1);
			sl->Add(s);

			Form2->StringGrid1->RowCount = Form2->StringGrid1->RowCount + 1;
			Form2->StringGrid1->Rows[Form2->StringGrid1->RowCount-1] = sl;
			}
		}

	if (Form2->StringGrid1->RowCount == 1)
		{
		Form2->Visible = false;
		MessageDlg("No JTAG devices found", mtError, TMsgDlgButtons() << mbOK, 0);
		}
	else
		{
		Form2->StringGrid1->FixedRows = 1;
		Form2->Visible = true;
		Form2->BringToFront();
		}
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SpeedButton1Click(TObject *Sender)
{
	updateGPIOs();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonProgramClick(TObject *Sender)
{
	Form3->ShowModal();
}
//---------------------------------------------------------------------------

static bool StatusCB(unsigned a, unsigned b)
{
	FormStatus->Set(a, b);
	return true;
}

//#include "TAPChain.h"

void __fastcall TForm1::BitBtn1Click(TObject *Sender)
{
	Timer1->Enabled = false;
	if (OpenDialogSVF->Execute())
		{
		std::ifstream input(OpenDialogSVF->FileName.c_str());
		FormStatus->Set(0, 10000);
		std::auto_ptr<JTAG::SVFPlayer> player(JTAG::SVFPlayer::new_SVFPlayer(*tm, port));
		Update();
		FormStatus->Visible = true;
		FormStatus->Update();

//		ChainInfo ci = {0, 0, 5, 0, 0};

		try
			{
			player->Play(input, 0, StatusCB,
						 true, !OpenDialogSVF->Options.Contains(ofReadOnly) );
			}

		catch (const JTAG::Error_Exception& err)
			{
			MessageDlg(err.what(), mtError, TMsgDlgButtons() << mbOK, 0);
			}

		FormStatus->Visible = false;
		}
	Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::BitBtn3Click(TObject *Sender)
{
	if (OpenDialogBIT->Execute())
		{
		std::ifstream input(OpenDialogBIT->FileName.c_str(),
							std::ios_base::in | std::ios_base::binary);
		FormStatus->Set(0, 10000);
		std::auto_ptr<JTAG::BITPlayer> player(JTAG::BITPlayer::new_BITPlayer(*tm, port));
		Update();
		FormStatus->Visible = true;
		FormStatus->Update();

//		ChainInfo ci = {0, 0, 5, 0, 0};

		try
			{
			player->Play(input, 0/*&ci*/, StatusCB);
			}

		catch (const JTAG::Error_Exception& err)
			{
			MessageDlg(err.what(), mtError, TMsgDlgButtons() << mbOK, 0);
			FormStatus->Visible = false;
			}

		FormStatus->Visible = false;
		}
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Report_Exception(TObject* Sender, Exception* err)
{
	Application->ShowException(err);
//	MessageDlg(err.what(), mtError, TMsgDlgButtons() << mbOK, 0);
}


